import javax.swing.JApplet;
import java.awt.Graphics;
import java.awt.Font;
import java.util.*;
import java.awt.event.*;
import java.awt.Component;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.util.concurrent.Semaphore;
import java.awt.Dimension;
import java.awt.GridLayout;

public class Applet extends JApplet implements Runnable, ActionListener
{
	Thread clock;
	scene s;
	String lastMoveByEngine = "";
	int nextMoveX = -2, nextMoveY = -2;
	boolean nextMoveValid = false;
	JTextField typingArea;
	JTextArea textArea;
	Semaphore available = new Semaphore(1, true);
	Semaphore drawingBoard = new Semaphore(1, true);
	player user = player.black;
	player currentPlayer = player.black;

	public void actionPerformed(ActionEvent evt)
	{
		String keyboardBuffer = typingArea.getText();
		typingArea.setText("");

		if (keyboardBuffer.equals("pass") || keyboardBuffer.equals("PASS"))
		{
			System.out.println("pass");
			nextMoveX = nextMoveY = -1;
			nextMoveValid = true;
			available.release();
			System.out.println("move valid");
		}
		else if (keyboardBuffer.length() >= 2 && s != null)
		{
			String moveString = keyboardBuffer;
			int d = s.getBoard().getDimensions();
			nextMoveX = moveString.charAt(0) - 'a';
			if (nextMoveX < 0)
				nextMoveX = moveString.charAt(0) - 'A';
			if (nextMoveX >= 8) // 'i' versus 'j'
				nextMoveX--;
			nextMoveY = Integer.valueOf(moveString.substring(1)) - 1;

			if (nextMoveX<0 || nextMoveX>=d || nextMoveY<0 || nextMoveY>=d)
			{
				nextMoveX = nextMoveY = -2;
			}
			else if (s.getBoard().getAt(nextMoveX, nextMoveY) == cross.empty)
			{
				nextMoveValid = true;
				available.release();
				System.out.println("move valid");
			}
		}
	}

	void showBoard(scene s)
	{
		final board b = s.getBoard();
		List<String> list = console.genBoard(b, true);

		textArea.setText(null);
		textArea.append("Stop v" + Version.getVersion() + ", (C) 2009-2015\nby folkert@vanheusden.com\n");
		for(String current : list)
			textArea.append(current + "\n");
		if (!lastMoveByEngine.equals(""))
			textArea.append("Last move by Stop: " + lastMoveByEngine + "\n");

		typingArea.requestFocus();

		if (currentPlayer == user)
		{
			if (nextMoveValid == false)
				textArea.append("Your move:");
		}
		else
			textArea.append("Thinking...");
	}

	public void paint(Graphics g)
	{
		try
		{
			drawingBoard.acquire();

			showBoard(s);

			drawingBoard.release();
		}
		catch(Exception e)
		{
			textArea.setText(null);
			textArea.append("Error: " + e);
		}
	}

	public String getAppletInfo()
	{
		return "Stop v" + Version.getVersion() + ", (C) folkert@vanheusden.com";
	}

	public void init()
	{
		Stop.initRandom();

		s = new scene(9, false);

		getContentPane().setLayout(new BorderLayout());

		final int windowWidth  = getSize().width;
		final int windowHeight = getSize().height;

		final int characterSize = 11;
		final Font f = new Font("Courier", Font.PLAIN, characterSize);

		final int nCharsWidth = windowWidth / characterSize;
		final int nCharsHeight = windowHeight / characterSize;

		System.out.println("window width: " + windowWidth + ", window height: " + windowHeight);
		System.out.println("# chars width: " + nCharsWidth + ", # chars height: " + nCharsHeight);

		textArea = new JTextArea(nCharsHeight - 1, nCharsWidth - 1);
		textArea.setEditable(false);
		textArea.setFont(f);
		getContentPane().add(textArea, BorderLayout.CENTER);

		typingArea = new JTextField(nCharsWidth);
		typingArea.setFont(f);
		getContentPane().add(typingArea, BorderLayout.SOUTH);
		typingArea.requestFocus();

		if (clock == null)
			clock = new Thread(this);
	}

	public void run()
	{
		while(clock != null)
		{
			try
			{
				int x, y;

				drawingBoard.acquire();
				showBoard(s);
				drawingBoard.release();

				if (currentPlayer == user)
				{
					System.out.println("acquire & addlistener");
					available.acquire();
					typingArea.addActionListener(this);
					System.out.println("acquire-2");
					available.acquire();
					System.out.println("remove & release");
					typingArea.removeActionListener(this);
					available.release();

					if (nextMoveValid)
					{
						s = s.doMove(nextMoveX, nextMoveY, currentPlayer == player.white ? cross.white:cross.black, currentPlayer);
						nextMoveValid = false;
						System.out.println("Did user move");
						textArea.append("" + Statics.genMoveString(nextMoveX, nextMoveY) + "\n");
					}
				}
				else
				{
					System.out.println("computer calculating");
					SelectedMove selectedMove = s.decideMove(currentPlayer, true, true, 1.0, new ArrayList<Output>(), 0);
					String moveStr = "pass";
					if (selectedMove != null)
					{
						x = selectedMove.getX();
						y = selectedMove.getY();

						moveStr = Statics.genMoveString(x, y);
						System.out.println("My move: " + moveStr);

						scene oldScene = s;
						s = s.doMove(x, y, currentPlayer == player.white ? cross.white:cross.black, currentPlayer);
						oldScene.destructor();
						oldScene = null;
					}
					System.out.println("finished");
					textArea.append("\nSelected move: " + moveStr + "\n");
					lastMoveByEngine = moveStr;
				}

				if (currentPlayer == player.white)
					currentPlayer = player.black;
				else
					currentPlayer = player.white;

			}
			catch(InterruptedException e)
			{
				System.out.println("User probably closed window");
				destroy();
				System.exit(127);
			}
			catch(Exception e)
			{
				System.out.println("Exception: " + e);
				System.out.println("Details: " + e.getMessage());
				System.out.println("Stack-trace:");
				for(StackTraceElement ste: e.getStackTrace())
				{
					System.out.println(" " + ste.getClassName() + ", "
							+ ste.getFileName() + ", "
							+ ste.getLineNumber() + ", "
							+ ste.getMethodName() + ", "
							+ (ste.isNativeMethod() ?
								"is native method" : "NOT a native method"));
				}
				destroy();
				System.exit(127);
			}
		}
	}

	public void start()
	{
		if (clock != null)
			clock.start();
	}

	public void stop()
	{
		Thread copy = clock;
		clock = null;
		copy.interrupt();
	}
}
