import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

abstract class Output {
	public Output() { }
	public abstract void close() throws IOException;
	public abstract void emitBoard(int moveNr, board b, SelectedMove [][] sm, chainpiece selected, player selectedColor) throws IOException;

	public BufferedImage drawBoard(int moveNr, board b, SelectedMove [][] sm, chainpiece selected, player thisColor) {
		// create canvas to draw on
		int dim = b.getDimensions();
		int left = 25, right = 25;
		int up = 25, down = 25;
		int stoneSize = 25, halfStone = stoneSize / 2;
		int width = b.getDimensions() * stoneSize + left + right;
		int height = b.getDimensions() * stoneSize + up + down;
		BufferedImage output = null;
		if (dim <= 9)
			output = new BufferedImage(352, 288, BufferedImage.TYPE_INT_RGB);
		else
			output = new BufferedImage(800, 600, BufferedImage.TYPE_INT_RGB);
		Graphics2D g = output.createGraphics();
		
		// set background
		g.setColor(Color.LIGHT_GRAY);
		if (dim <= 9)
			g.fillRect(0, 0, 352, 288);
		else
			g.fillRect(0, 0, 800, 600);
		
		// create a palette
// FIXME: use H from HLS with S/L 0.5
		Color [] pal = new Color[512];
		for(int rii=0; rii<450; rii++) {
			int ri = (int)(rii * (512.0 / 450.0));
			int r_val = 16;
			int g_val = ri >> 1;
			int b_val = 128 + 4 * (ri & 1);
			pal[rii] = new Color((r_val << 16) + (g_val << 8) + b_val);
		}
		for(int rii=450; rii<512; rii++) {
			int ri = (int)((rii - 450) * (192.0 / 128.0));
			int r_val = 128 + ri;
			int g_val = 16;
			int b_val = 16;
			pal[rii] = new Color((r_val << 16) + (g_val << 8) + b_val);
		}

		// draw raster
		g.setColor(Color.BLACK);
		for(int d=0; d<dim; d++) {
			g.drawLine(left, up + d * stoneSize, width - right - stoneSize, up + d * stoneSize);
			g.drawLine(left + d * stoneSize, up, left + d * stoneSize, height - down - stoneSize);
		}

		// draw move string
		String moveStr = "" + moveNr + ". ";
		if (selected != null)
			moveStr += Statics.genMoveString(selected.getX(), selected.getY());
		else
			moveStr += "pass";
		moveStr += " " + thisColor;
		Font font = new Font("Arial", Font.PLAIN, 27);
		g.setFont(font);
		if (dim <= 9)
			g.drawString(moveStr, 10, 262);
		else
			g.drawString(moveStr, 10, 575);
		font = new Font("Arial", Font.PLAIN, 10);
		g.setFont(font);
		g.drawString("Stop v" + Version.getVersion() + ", (C) folkert@vanheusden.com", 10, 9);
		
		// draw old stones
		for(int y=0; y<dim; y++) {
			for(int x=0; x<dim; x++) {
				cross c = b.getAt(x, y);
				if (c == cross.white)
					g.setColor(Color.WHITE);
				else
					g.setColor(Color.BLACK);
				if (c == cross.white || c == cross.black)
				{
					g.fillOval(left + x * stoneSize - halfStone, up + y * stoneSize - halfStone, stoneSize, stoneSize);
					g.setColor(Color.BLACK);
					g.drawOval(left + x * stoneSize - halfStone, up + y * stoneSize - halfStone, stoneSize, stoneSize);
				}
			}
		}
		
		// determine min/max of selected move values
		double max_val = -999999999999.0, min_val = -max_val;
		for(int y=0; y<dim; y++) {
			for(int x=0; x<dim; x++) {
				if (sm[x][y].isValidMove() == false || sm[x][y].getCount() == 0)
					continue;

				max_val = Math.max(max_val, sm[x][y].getValue());
				min_val = Math.min(min_val, sm[x][y].getValue());
			}
		}
					
		// draw selected moves with color set to their value
		for(int y=0; y<dim; y++) {
			for(int x=0; x<dim; x++) {
				if (sm[x][y].isValidMove() == false || sm[x][y].getCount() == 0)
					continue;

				if (x == selected.getX() && y == selected.getY())
					g.setColor(Color.GRAY);
				else {
					double cur_val = sm[x][y].getValue();
					double val_norm = (cur_val - min_val) / (max_val - min_val);
					int pal_index = (int)(val_norm * 511);
					if (pal_index < 0)
						pal_index = 0;
					if (pal_index > 511)
						pal_index = 511;
					g.setColor(pal[pal_index]);
				}
				
				g.fillOval(left + x * stoneSize - halfStone, up + y * stoneSize - halfStone, stoneSize, stoneSize);
				g.setColor(Color.BLACK);
				g.drawOval(left + x * stoneSize - halfStone, up + y * stoneSize - halfStone, stoneSize, stoneSize);
			}
		}

		return output;
	}
}
