import java.util.ArrayList;
import java.util.List;
import java.util.Random;

final class Stop {
	static Random random = null;

	public static player opponentColor(player who) {
		if (who == null || who == player.white)
			return player.black;

		return player.white;
	}

	public static boolean randomTrue() {
		return Math.abs(random.nextInt(10)) < 5.0;
	}

	public static int genRandom(int max_value) {
		return Math.abs(random.nextInt(max_value));
	}

	public static double genRandomDouble() {
		return random.nextDouble();
	}

	public static void initRandom() {
		random = new Random();
	}

	public static void main(String [] args) throws Exception {
		boolean random_allowed = false;
		Mode current_mode = Mode.console;
		int board_size = 19;
		String log_file = null;
		double thinkTime = 1.0;
		boolean deep = false;
		List<Output> outputs = new ArrayList<Output>();

		for(int argIndex=0; argIndex<args.length; argIndex++) {
			if (args[argIndex].compareTo("--mode") == 0) {
				String new_mode = args[++argIndex];

				if (new_mode.equals("console"))
					current_mode = Mode.console;
				else if (new_mode.equals("gtp"))
					current_mode = Mode.gtp;
				else if (new_mode.equals("test"))
					current_mode = Mode.test;
				else {
					System.out.println("--mode requires either console, gtp or test as parameter");
					System.exit(1);
				}
			}
			else if (args[argIndex].compareTo("--verbose") == 0) {
				Statics.setVerbose(true);
			}
			else if (args[argIndex].compareTo("--output") == 0) {
				String type = args[++argIndex];
				String file = args[++argIndex];

				if (type.equalsIgnoreCase("mjpeg"))
					outputs.add(new OutputMjpeg(file));
				else if (type.equalsIgnoreCase("avi"))
					outputs.add(new OutputAvi(file));
//					else if (type.equalsIgnoreCase("sgf"))
//						outputs.add(new OutputSgf(file));
				else {
					System.out.println("--output with type '" + type + "' is not supported");
					System.exit(1);
				}
			}
			else if (args[argIndex].compareTo("--logfile") == 0) {
				log_file = args[++argIndex];
			}
			else if (args[argIndex].compareTo("--boardsize") == 0) {
				board_size = Integer.valueOf(args[++argIndex]);
			}
			else if (args[argIndex].compareTo("--time") == 0) {
				thinkTime = Double.valueOf(args[++argIndex]);
			}
			else if (args[argIndex].compareTo("--brainsettings-map") == 0) {
				MaxField.setBrainSettings(args[++argIndex]);
			}
			else if (args[argIndex].compareTo("--random-allowed") == 0 || args[argIndex].compareTo("--allow-random") == 0) {
				random_allowed = true;
			}
			else if (args[argIndex].compareTo("--deep") == 0) {
				deep = true;
			}
			else {
				System.out.println("Parameter '" + args[argIndex] + "' not understood.");
				System.exit(1);
			}
		}

		Statics.setLogFile(log_file);

		initRandom();

		if (current_mode == Mode.console)
			console.loop(board_size, random_allowed, thinkTime, deep, outputs);
		else if (current_mode == Mode.gtp)
			gtp.loop(board_size, random_allowed, thinkTime, deep, outputs);
		else if (current_mode == Mode.test)
			test.loop(board_size, random_allowed, thinkTime, deep, outputs);

		for(Output o : outputs)
			o.close();

		System.exit(0);
	}
}
