import java.util.*;

final class board {
	final cross [] b;
	final int dimensions, nFields;
	boolean test;

	board(int dimensions, boolean is_test) {
		assert dimensions > 0;
		nFields = dimensions * dimensions;

		b = new cross[nFields];
		Arrays.fill(b, cross.empty);

		this.dimensions = dimensions;
		test = is_test;
	}

	board(board bIn) {
		nFields = bIn.getNFields();

		dimensions = bIn.getDimensions();

		b = bIn.getElements().clone();
	}

	protected cross [] getElements() {
		return b;
	}

	int getNFields() {
		return nFields;
	}

	int getHash() {
		int out = 0;
		int shift = 0;

		for(int y=0; y<dimensions; y++) {
			for(int x=0; x<dimensions; x++) {
				if (getAt(x, y) == cross.white)
					out ^= (1 << shift);
				else if (getAt(x, y) == cross.black)
					out |= (2 << shift);

				shift += 2;

				if (shift >= 32)
					shift = 0;
			}
		}

		return out;
	}

	int countObjects(cross type) {
		int count = 0;

		for(int index=0; index<nFields; index++) {
			if (getAt(index) == type)
				count++;
		}

		return count;
	}

	public boolean equals(Object other) {
		board compareWith = (board)other;

		if (getDimensions() != compareWith.getDimensions())
			return false;

		for(int index=0; index<nFields; index++) {
			if (getAt(index) != compareWith.getAt(index))
				return false;
		}

		assert getHash() == compareWith.getHash();

		return true;
	}

	board duplicateBoard() {
		return new board(this);
	}

	board doMove(int x, int y, cross type) {
		assert x >=0 && y>=0 && x < dimensions && y < dimensions;
		assert type == cross.white || type == cross.black;

		board new_board = duplicateBoard();

		new_board.putAt(x, y, type);

		return new_board;
	}

	cross getAt(int x, int y) {
		assert x >=0 && y>=0 && x < dimensions && y < dimensions;
		return b[y * dimensions + x];
	}

	cross getAt(int index) {
		return b[index];
	}

	void clear(int x, int y) {
		assert b[y * dimensions + x] != cross.empty;

		b[y * dimensions + x] = cross.empty;
	}

	void putAt(int x, int y, cross obj) {
		assert x >=0 && y>=0 && x < dimensions && y < dimensions;

		if (obj == cross.white || obj == cross.black)
			assert b[y * dimensions + x] == cross.empty;

		b[y * dimensions + x] = obj;
	}

	void putAt(int index, cross obj) {
		if (obj == cross.white || obj == cross.black)
			assert b[index] == cross.empty;

		b[index] = obj;
	}

	int getDimensions() {
		return dimensions;
	}

	int countObjectsAroundXYOfType(int x, int y, cross type) {
		int count = 0;

		if (x > 0                     && getAt(x - 1, y) == type)
			count++;

		if (x < (getDimensions() - 1) && getAt(x + 1, y) == type)
			count++;

		if (y > 0                     && getAt(x, y - 1) == type)
			count++;

		if (y < (getDimensions() - 1) && getAt(x, y + 1) == type)
			count++;

		return count;
	}
}
