import java.util.*;

final class chain {
	final board b;
	final player color;
	List<chainpiece> chainelements;
	List<chainpiece> freedoms;

	// color: color of player who owns these pieces
	chain(board b, player color) {
		this.b = b;
		this.color = color;
		chainelements = new ArrayList<chainpiece>();
		freedoms = null;
	}

	void destructor() {
		if (chainelements != null) {
			chainelements.clear();
			chainelements = null;
		}

		if (freedoms != null) {
			freedoms.clear();
			freedoms = null;
		}
	}

	protected void finalize() throws Throwable {
		super.finalize();
	}

	player getColor() {
		return color;
	}

	List<chainpiece> getList() {
		return chainelements;
	}

	int size() {
		return chainelements.size();
	}

	boolean isInThisChain(int x, int y) {
		return Statics.isInThisChain(chainelements, x, y);
	}

	String dump() {
		String out = "";
		out += "Chain: ";
		if (color == player.white)
			out += "white ";
		else
			out += "black ";
		for(chainpiece current : chainelements) {
			out += " " + current.getX() + "," + current.getY();
		}
		out += ", freedoms: " + getFreedoms().size();

		return out;
	}

	boolean doesPieceConnectToChain(int x, int y) {
		assert x>=0 && y>=0 && x<b.getDimensions() && y<b.getDimensions();
		final int xLeft = x - 1, xRight = x + 1;
		final int yUp = y - 1, yDown = y + 1;

		for(chainpiece current : chainelements) {
			// TEST
			if (current.getX() == x && (current.getY() == yUp   || current.getY() == yDown))
				return true;
			if (current.getY() == y && (current.getX() == xLeft || current.getX() == xRight))
				return true;
		}

		return false;
	}

	void addPiece(int x, int y) {
		assert x>=0 && y>=0 && x<b.getDimensions() && y<b.getDimensions();

		chainelements.add(new chainpiece(x, y));
	}

	void addPiece(chainpiece cp) {
		assert cp != null;

		chainelements.add(cp);
	}

	static void addToChainpieceListKeepUnique(List<chainpiece> list, int x, int y) {
		assert x>=0 && y>=0;

		if (!Statics.isInThisChain(list, x, y))
			list.add(new chainpiece(x, y));
	}

	public class ChainPieceComparator implements Comparator<chainpiece> {
		public int compare(chainpiece o1, chainpiece o2) {
			return o1.compareTo(o2);
		}
	}

	void merge(final chain source_in) {
		final List<chainpiece> source = source_in.getList();

		chainelements.removeAll(source);
		chainelements.addAll(source);
	}

	void merge(final List<chainpiece> target, final List<chainpiece> source) {
		target.removeAll(source);
		target.addAll(source);
	}

	List<chainpiece> getFreedoms() {
		if (freedoms == null) {
			final int dimm = b.getDimensions() - 1;

			freedoms = new ArrayList<chainpiece>();

			for(final chainpiece current : chainelements) {
				final int x = current.getX();
				final int y = current.getY();

				if (x > 0    && b.getAt(x - 1, y) == cross.empty && !Statics.isInThisChain(freedoms, x - 1, y))
					freedoms.add(new chainpiece(x - 1, y));

				if (x < dimm && b.getAt(x + 1, y) == cross.empty && !Statics.isInThisChain(freedoms, x + 1, y))
					freedoms.add(new chainpiece(x + 1, y));

				if (y > 0    && b.getAt(x, y - 1) == cross.empty && !Statics.isInThisChain(freedoms, x, y - 1))
					freedoms.add(new chainpiece(x, y - 1));

				if (y < dimm && b.getAt(x, y + 1) == cross.empty && !Statics.isInThisChain(freedoms, x, y + 1))
					freedoms.add(new chainpiece(x, y + 1));
			}
		}

		return freedoms;
	}

	void resetFreedomsList() {
		freedoms = null;
	}
}
